/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This example demonstrates how you can build your own ASN.1 compiler
 * control table DLL exporting all the external variables/constants,
 * load it and access those external variables/constants.  The following
 * APIs are used in this example:
 *
 * ossWinit()
 * ossPrint()
 * ossPrintPDU()
 * ossEncode()
 * ossDecode()
 * ossFreeBuf()
 * ossFreePDU()
 * ossWterm()
 *
 * The feature demonstrated in this example is only available starting
 * from OSS ASN.1 Tools for C 6.0.0 Beta C or later.
 */



#define OSS_EXPORT_DLL_DATA __declspec(dllimport)
#include "dllexp.h"


int main(void)
{

    OssGlobal ww, *world = &ww;
    int       pdunum = 0;
    int       rc = 0;
    OssBuf    encodedData;
    void     *decodedData;


    rc = ossWinit(world, NULL, "dll.dll", NULL);
    if (rc) {
        ossPrint(NULL, "%d was returned by ossinit().\n", rc);
        return (rc? EXIT_FAILURE: EXIT_SUCCESS);
    }

    ossPrint(world, "TESTING DLL EXPORTED CONSTANTS/VARIABLES...\n\n");

    ossSetFlags(world, DEBUGPDU);
    encodedData.length = 0;
    encodedData.value  = NULL;
    pdunum = Age_PDU;
    decodedData = NULL;

    ossPrint(world, "\n\tUnencoded PDU...\n");
    if (ossPrintPDU(world, pdunum, (void *)&myage))
	rc++;

    ossPrint(world, "\n\tTracing information from the encoder...\n");
    if (ossEncode(world, pdunum, (void *)&myage, &encodedData))
	rc++;
    else {
	ossPrint(world, "\n\tTracing information from the decoder...\n");
	if (ossDecode(world, &pdunum, &encodedData, &decodedData)) {
	    rc++;
	    ossFreeBuf(world, encodedData.value);
	}
	else {
	    ossPrint(world, "\n\tDecoded PDU...\n");
	    ossPrintPDU(world, pdunum, decodedData);
	    ossFreePDU(world, pdunum, decodedData);
	}
    }

    encodedData.length = 0;
    encodedData.value  = NULL;
    pdunum = Personal_PDU;
    decodedData = NULL;

    ossPrint(world, "\n\n\tUnencoded PDU...\n");
    if (ossPrintPDU(world, pdunum, &details))
	rc++;

    ossPrint(world, "\n\tTracing information from the encoder...\n");
    if (ossEncode(world, pdunum, &details, &encodedData))
	rc++;
    else {
	ossPrint(world, "\n\tTracing information from the decoder...\n");
	if (ossDecode(world, &pdunum, &encodedData, &decodedData)) {
	    rc++;
	    ossFreeBuf(world, encodedData.value);
	}
	else {
	    ossPrint(world, "\n\tDecoded PDU...\n");
	    ossPrintPDU(world, pdunum, decodedData);
	    ossFreePDU(world, pdunum, decodedData);
	}
    }

    encodedData.length = 0;
    encodedData.value  = NULL;
    pdunum = Perchoice_PDU;
    decodedData = NULL;

    ossPrint(world, "\n\n\tUnencoded PDU...\n");
    if (ossPrintPDU(world, pdunum, &addition))
	rc++;

    ossPrint(world, "\n\tTracing information from the encoder...\n");
    if (ossEncode(world, pdunum, &addition, &encodedData))
	rc++;
    else {
	ossPrint(world, "\n\tTracing information from the decoder...\n");
	if (ossDecode(world, &pdunum, &encodedData, &decodedData)) {
	    rc++;
	    ossFreeBuf(world, encodedData.value);
	}
	else {
	    ossPrint(world, "\n\tDecoded PDU...\n");
	    ossPrintPDU(world, pdunum, decodedData);
	    ossFreePDU(world, pdunum, decodedData);
	}
    }

    ossPrint(world, rc? "Test failed.\n": "Test passed.\n");

    ossWterm(world);
    return (rc? EXIT_FAILURE: EXIT_SUCCESS);
}
